/*
 *	Copyright (C) 2009 Jonathan Naylor, G4KLX
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 */

#if !defined(__DSTARRADIOHEADERPROCESSOR_H)
#define	__DSTARRADIOHEADERPROCESSOR_H

#include "DStarDefines.h"
#include "DStarScrambler.h"

#include "Utils.h"

const unsigned int LONG_CALLSIGN_LENGTH  = 8U;
const unsigned int SHORT_CALLSIGN_LENGTH = 4U;

enum DSRH_FLAGS {
	DSRH_NONE,
	DSRH_RELAY_UNAVAILABLE,
	DSRH_NO_RESPONSE,
	DSRH_ACK_FLAG,
	DSRH_REQUEST_RESEND,
	DSRH_AUTO_REPLY,
	DSRH_REPEATER_CONTROL
};

class CDStarRadioHeaderProcessor {
public:
	CDStarRadioHeaderProcessor();
	virtual ~CDStarRadioHeaderProcessor();

	virtual bool         setRadioData(CDStarScrambler& scrambler, const bool* data, unsigned int length);
	virtual unsigned int getHeaderData(bool* data, unsigned int length);
	virtual unsigned int getRadioData(CDStarScrambler& scrambler, bool* data, unsigned int length);

	virtual void setOwnCallsign1(const char* call);
	virtual char* getOwnCallsign1();

	virtual void setOwnCallsign2(const char* call);
	virtual char* getOwnCallsign2();

	virtual void setCompanionCallsign(const char* call);
	virtual char* getCompanionCallsign();

	virtual void setDestinationRepeaterCallsign(const char* call);
	virtual char* getDestinationRepeaterCallsign();

	virtual void setDepartureRepeaterCallsign(const char* call);
	virtual char* getDepartureRepeaterCallsign();

	virtual void setDirectMode(bool set);
	virtual bool isDirectMode() const;

	virtual void setDataPacket(bool set);
	virtual bool isDataPacket() const;

	virtual void setInterrupted(bool set);
	virtual bool isInterrupted() const;

	virtual void setControlSignal(bool set);
	virtual bool isControlSignal() const;

	virtual void setUrgent(bool set);
	virtual bool isUrgent() const;

	virtual void setRepeaterFlags(DSRH_FLAGS set);
	virtual DSRH_FLAGS getRepeaterFlags() const;

	virtual void debug(bool set);
	virtual void reset();

private:
	bool       m_header[RADIO_HEADER_LENGTH_BITS];
	char       m_ownCallsign1[10U];
	char       m_ownCallsign2[10U];
	char       m_companionCallsign[10U];
	char       m_destinationRepeaterCallsign[10U];
	char       m_departureRepeaterCallsign[10U];
	bool       m_direct;
	bool       m_data;
	bool       m_interrupted;
	bool       m_control;
	bool       m_urgent;
	DSRH_FLAGS m_flags;
	bool       m_debug;

	void setCallsign(char* callOut, const char* callIn, unsigned int length);
};

#endif
