/*
 *	Copyright (C) 2009 Jonathan Naylor, G4KLX
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 */

#include "Utils.h"

#include <cstdio>
#include <cctype>
#include <cassert>

void CUtils::dump(char* title, const bool* data, unsigned int length)
{
	assert(title != 0);
	assert(data != 0);

	::fprintf(stdout, "%s\n", title);

	unsigned int offset = 0U;

	while (offset < length) {
		::fprintf(stdout, "%04X:  ", offset / 8U);

		unsigned char buffer[16];

		unsigned int bytes = 0U;
		for (unsigned int bits = 0U; bits < 128U && (offset + bits) < length; bits += 8U)
			buffer[bytes++] = bitsToByte(data + offset + bits);

		for (unsigned i = 0U; i < bytes; i++)
			::fprintf(stdout, "%02X ", buffer[i]);

		for (unsigned int i = bytes; i < 16U; i++)
			::fprintf(stdout, "   ");

		::fprintf(stdout, "   *");

		for (unsigned i = 0U; i < bytes; i++) {
			unsigned char c = buffer[i];

			if (::isprint(c))
				::fprintf(stdout, "%c", c);
			else
				::fprintf(stdout, ".");
		}

		::fprintf(stdout, "*\n");

		offset += 128U;
	}

	::fprintf(stdout, "\n");
}

void CUtils::dumpRev(char* title, const bool* data, unsigned int length)
{
	assert(title != 0);
	assert(data != 0);

	::fprintf(stdout, "%s\n", title);

	unsigned int offset = 0U;

	while (offset < length) {
		::fprintf(stdout, "%04X:  ", offset / 8U);

		unsigned char buffer[16];

		unsigned int bytes = 0U;
		for (unsigned int bits = 0U; bits < 128U && (offset + bits) < length; bits += 8U)
			buffer[bytes++] = bitsToByteRev(data + offset + bits);

		for (unsigned i = 0U; i < bytes; i++)
			::fprintf(stdout, "%02X ", buffer[i]);

		for (unsigned int i = bytes; i < 16U; i++)
			::fprintf(stdout, "   ");

		::fprintf(stdout, "   *");

		for (unsigned i = 0U; i < bytes; i++) {
			unsigned char c = buffer[i];

			if (::isprint(c))
				::fprintf(stdout, "%c", c);
			else
				::fprintf(stdout, ".");
		}

		::fprintf(stdout, "*\n");

		offset += 128U;
	}

	::fprintf(stdout, "\n");
}

void CUtils::dumpRev(char* title, const unsigned char* data, unsigned int length)
{
	assert(title != 0);
	assert(data != 0);

	::fprintf(stdout, "%s\n", title);

	unsigned int offset = 0U;

	while (length > 0U) {
		::fprintf(stdout, "%04X:  ", offset);

		unsigned int bytes = (length > 16U) ? 16U : length;

		for (unsigned i = 0U; i < bytes; i++)
			::fprintf(stdout, "%02X ", data[offset + i]);

		for (unsigned int i = bytes; i < 16U; i++)
			::fprintf(stdout, "   ");

		::fprintf(stdout, "   *");

		for (unsigned i = 0U; i < bytes; i++) {
			unsigned char c = data[offset + i];

			if (::isprint(c))
				::fprintf(stdout, "%c", c);
			else
				::fprintf(stdout, ".");
		}

		::fprintf(stdout, "*\n");

		offset += 16U;

		if (length >= 16U)
			length -= 16U;
		else
			length = 0U;
	}

	::fprintf(stdout, "\n");
}

unsigned char CUtils::bitsToByte(const bool* bits)
{
	assert(bits != 0);

	unsigned char val = 0x00;

	for (unsigned int i = 0U; i < 8U; i++) {
		val <<= 1;

		if (bits[i])
			val |= 0x01;
	}

	return val;
}

unsigned char CUtils::bitsToByteRev(const bool* bits)
{
	assert(bits != 0);

	unsigned char val = 0x00;

	for (unsigned int i = 0U; i < 8U; i++) {
		val >>= 1;

		if (bits[i])
			val |= 0x80;
	}

	return val;
}

void CUtils::byteToBits(unsigned char byte, bool* data)
{
	assert(data != 0);

	unsigned char mask = 0x80U;
	for (unsigned int i = 0U; i < 8U; i++, mask >>= 1)
		data[i] = byte & mask ? true : false;
}

void CUtils::byteToBitsRev(unsigned char byte, bool* data)
{
	assert(data != 0);

	unsigned char mask = 0x01U;
	for (unsigned int i = 0U; i < 8U; i++, mask <<= 1)
		data[i] = byte & mask ? true : false;
}
